package net.projectmonkey.object.mapper.analysis.duplicates;

/*
 *
 *  * Copyright 2012 the original author or authors.
 *  *
 *  * Licensed under the Apache License, Version 2.0 (the "License");
 *  * you may not use this file except in compliance with the License.
 *  * You may obtain a copy of the License at
 *  *
 *  *      http://www.apache.org/licenses/LICENSE-2.0
 *  *
 *  * Unless required by applicable law or agreed to in writing, software
 *  * distributed under the License is distributed on an "AS IS" BASIS,
 *  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  * See the License for the specific language governing permissions and
 *  * limitations under the License.
 *
 */

import net.projectmonkey.object.mapper.analysis.result.PropertyPath;

import java.util.Collection;
import java.util.List;

/**
 * Implementations define how to handle multiple source / destination paths found by an
 * {@link net.projectmonkey.object.mapper.analysis.matching.MatchingStrategy} for a single corresponding
 * source / destination path.
 *
 *  N.B. Implementations of DuplicateMappingBehaviour should be referenced statically
 * or define equals in a way which means that separate instances will
 * be considered equal if the output from them will be the same.
 *
 * This is required since it forms part of the {@link net.projectmonkey.object.mapper.analysis.cache.MappingsCacheKey}.
 *
 * @author Andy Moody
 */
public interface DuplicateMappingBehaviour
{

	/**
	 * @param sourcePath
	 * @param matchingDestinations
	 * @return the matchingDestinations filtered according to the implementation
	 * @throws IllegalStateException if the duplicates cannot be resolved.
	 */
	<T extends PropertyPathAndTokens> List<PropertyPath> filterDestinationsBySource(T sourcePath, Collection<T> matchingDestinations);

	/**
	 * @param destinationPath
	 * @param matchingSources
	 * @return the matchingSources filtered according to the implementation
	 * @throws IllegalStateException if the duplicates cannot be resolved.
	 */
	<T extends PropertyPathAndTokens> PropertyPath filterSourcesByDestination(T destinationPath, Collection<T> matchingSources);

}
