package net.projectmonkey.object.mapper.analysis.matching;

import net.projectmonkey.object.mapper.analysis.result.PropertyMapping;
import net.projectmonkey.object.mapper.analysis.tokenizer.PropertyNameTokenizer;
import net.projectmonkey.object.mapper.context.ConversionConfiguration;
import net.projectmonkey.object.mapper.context.ExecutionContext;
import net.projectmonkey.object.mapper.analysis.cache.TypePair;
import net.projectmonkey.object.mapper.analysis.result.PropertyPath;

import java.util.ArrayList;
import java.util.List;

/*
 *
 *  * Copyright 2012 the original author or authors.
 *  *
 *  * Licensed under the Apache License, Version 2.0 (the "License");
 *  * you may not use this file except in compliance with the License.
 *  * You may obtain a copy of the License at
 *  *
 *  *      http://www.apache.org/licenses/LICENSE-2.0
 *  *
 *  * Unless required by applicable law or agreed to in writing, software
 *  * distributed under the License is distributed on an "AS IS" BASIS,
 *  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  * See the License for the specific language governing permissions and
 *  * limitations under the License.
 *
 */

/**
 * Matching strategy which will only map properties if the
 * paths to them are exactly the same between the source
 * and destination object structures.
 *
 * @author Andy Moody
 */
public class ExactMatchingStrategy implements MatchingStrategy
{

	public static ExactMatchingStrategy INSTANCE = new ExactMatchingStrategy();

	ExactMatchingStrategy()
	{
	}

	@Override
	public List<PropertyMapping> resolveMatches(final TypePair<?, ?> types, final List<PropertyPath> sourcePaths, final List<PropertyPath> destinationPaths)
	{

		List<PropertyMapping> mappings = ExecutionContext.getMappings(types);

		if(mappings == null)
		{
			mappings = new ArrayList<PropertyMapping>();
			ExecutionContext.registerMappings(types, mappings);
			ConversionConfiguration configuration = getConfiguration();

			PropertyNameTokenizer sourcePathTokenizer = configuration.getSourceTokenizer();
			PropertyNameTokenizer destinationPathTokenizer = configuration.getDestinationTokenizer();
			for(PropertyPath sourcePath : sourcePaths)
			{
				List<String> sourceTokens = sourcePathTokenizer.tokenizePath(sourcePath.getProperty());
				for(PropertyPath destinationPath : destinationPaths)
				{
					List<String> destTokens = destinationPathTokenizer.tokenizePath(destinationPath.getProperty());
					if(sourceTokens.equals(destTokens))
					{
						TypePair propertyTypes = TypePair.of(sourcePath.getProperty().getType(), destinationPath.getProperty().getType());
						List<PropertyMapping> childMappings = resolveMatches(propertyTypes, sourcePath.getChildren(), destinationPath.getChildren());
						PropertyMapping mapping = new PropertyMapping(sourcePath, destinationPath, childMappings);
						mappings.add(mapping);
					}
				}
			}
		}
		return mappings;
	}

	protected ConversionConfiguration getConfiguration()
	{
		return ExecutionContext.getConfiguration();
	}

}
