package net.projectmonkey.object.mapper.analysis.token.matcher;

/*
 *
 *  * Copyright 2012 the original author or authors.
 *  *
 *  * Licensed under the Apache License, Version 2.0 (the "License");
 *  * you may not use this file except in compliance with the License.
 *  * You may obtain a copy of the License at
 *  *
 *  *      http://www.apache.org/licenses/LICENSE-2.0
 *  *
 *  * Unless required by applicable law or agreed to in writing, software
 *  * distributed under the License is distributed on an "AS IS" BASIS,
 *  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  * See the License for the specific language governing permissions and
 *  * limitations under the License.
 *
 */

/**
 * @author Andy Moody
 */

import java.util.Arrays;
import java.util.List;

public class MatchingSummary{

	private final List<TokenMatch> matches;
	private final int propertyPosition;
	private final List<String> unmatchedDestinationTokens;

	public MatchingSummary(final List<TokenMatch> matches, final List<String> unmatchedDestinationTokens, final int propertyPosition) {
		this.matches = matches;
		this.propertyPosition = propertyPosition;
		this.unmatchedDestinationTokens = unmatchedDestinationTokens;
	}

	public MatchingSummary(final TokenMatch match, final List<String> unmatchedDestinationTokens, final int propertyPosition)
	{
		this(Arrays.asList(match), unmatchedDestinationTokens, propertyPosition);
	}

	public int getScore()
	{
		int score = 0;
		for (TokenMatch match : matches)
		{
			score -= match.getStrength().getPriority();
		}
		for(String unmatchedDestinationToken : unmatchedDestinationTokens)
		{
			score -= MatchStrength.NONE.getPriority();
		}
		return score;
	}

	public boolean isMatch()
	{
		for(TokenMatch match : matches)
		{
			if(match !=null && match.getStrength() != null && match.getStrength() != MatchStrength.NONE)
			{
				return true;
			}
		}
		return false;
	}

	public List<TokenMatch> getMatches()
	{
		return matches;
	}

	public int getPropertyPosition()
	{
		return propertyPosition;
	}

	public List<String> getUnmatchedDestinationTokens()
	{
		return unmatchedDestinationTokens;
	}
}
