package net.projectmonkey.object.mapper.analysis.tokenizer;

import net.projectmonkey.object.mapper.analysis.result.PropertyPathElement;

import java.util.Arrays;
import java.util.List;
import java.util.regex.Pattern;

/*
 *
 *  * Copyright 2012 the original author or authors.
 *  *
 *  * Licensed under the Apache License, Version 2.0 (the "License");
 *  * you may not use this file except in compliance with the License.
 *  * You may obtain a copy of the License at
 *  *
 *  *      http://www.apache.org/licenses/LICENSE-2.0
 *  *
 *  * Unless required by applicable law or agreed to in writing, software
 *  * distributed under the License is distributed on an "AS IS" BASIS,
 *  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  * See the License for the specific language governing permissions and
 *  * limitations under the License.
 *
 */

/**
 * Adapted from Modelmapper (modelmapper.org)
 *
 * @author Jonathan Halterman
 * @author Andy Moody
 */
public class CamelCaseTokenizer implements PropertyNameTokenizer
{

	public static final CamelCaseTokenizer INSTANCE = new CamelCaseTokenizer();

	private CamelCaseTokenizer()
	{

	}

	private static final Pattern camelCase = Pattern
			.compile("(?<=[A-Z])(?=[A-Z][a-z])|(?<=[^A-Z])(?=[A-Z])|(?<=[A-Za-z])(?=[^A-Za-z])");

	@Override
	public List<String> tokenizePath(final PropertyPathElement element)
	{
		return Arrays.asList(camelCase.split(element.getName()));
	}

}
