package net.projectmonkey.object.mapper.construction;

/*
 *
 *  * Copyright 2012 the original author or authors.
 *  *
 *  * Licensed under the Apache License, Version 2.0 (the "License");
 *  * you may not use this file except in compliance with the License.
 *  * You may obtain a copy of the License at
 *  *
 *  *      http://www.apache.org/licenses/LICENSE-2.0
 *  *
 *  * Unless required by applicable law or agreed to in writing, software
 *  * distributed under the License is distributed on an "AS IS" BASIS,
 *  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  * See the License for the specific language governing permissions and
 *  * limitations under the License.
 *
 */

/**
 * @author Andy Moody
 */
public interface ObjectConstructionMechanism
{

	/**
	 * Construct a destination from the given context.
	 * @param type
	 * @param context
	 * @param <T>
	 * @return the constructed object
	 */
	<T> T construct(Class<T> type, PopulationContext<?, ?> context);

    /**
	 * Populate the given destination from the provided mechanism.
	 * @param destination
	 * @param context
	 * @param <T>
	 */
	<T> void populate(final Object destination, final PopulationContext context);

	/**
	 * Define whether we always need to construct a new object - e.g. for immutable objects.
	 * @param context
	 * @return true if the construct method should be called when a destination already exists, false otherwise.
	 */
	boolean alwaysConstruct(PopulationContext context);
}
