package net.projectmonkey.object.mapper.construction.converter;

import net.projectmonkey.object.mapper.construction.PopulationContext;
import net.projectmonkey.object.mapper.construction.RootPopulationContext;

/*
 *
 *  * Copyright 2012 the original author or authors.
 *  *
 *  * Licensed under the Apache License, Version 2.0 (the "License");
 *  * you may not use this file except in compliance with the License.
 *  * You may obtain a copy of the License at
 *  *
 *  *      http://www.apache.org/licenses/LICENSE-2.0
 *  *
 *  * Unless required by applicable law or agreed to in writing, software
 *  * distributed under the License is distributed on an "AS IS" BASIS,
 *  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  * See the License for the specific language governing permissions and
 *  * limitations under the License.
 *
 */

/**
 *
 * Converter which seeks to create destinations of type Enum
 * based on the toString of the source object.
 *
 * Adapted from ModelMapper (modelmapper.org)
 *
 * @author Jonathan Halterman
 * @author Andy Moody
 */
public class EnumConverter implements Converter<Object, Enum>
{
	public static final EnumConverter INSTANCE = new EnumConverter();

	@Override
	public Enum convert(final PopulationContext<Object, Enum> context)
	{
		Enum toReturn = null;
		Object source = context.getSource();
		Class<? extends Enum> destinationType = context.getDestinationType();
		if(source != null)
		{

			PopulationContext<Object, String> stringConversionContext = new RootPopulationContext<Object, String>(source, String.class);
			String stringRepresentation = StringConverter.INSTANCE.convert(stringConversionContext);
			toReturn = Enum.valueOf(destinationType, stringRepresentation);
		}
		return toReturn;
	}

	@Override
	public boolean canConvert(final PopulationContext<?,?> context)
	{
		return context.getDestinationType().isEnum();
	}

}
