package net.projectmonkey.object.mapper.construction.converter;

/*
 *
 *  * Copyright 2012 the original author or authors.
 *  *
 *  * Licensed under the Apache License, Version 2.0 (the "License");
 *  * you may not use this file except in compliance with the License.
 *  * You may obtain a copy of the License at
 *  *
 *  *      http://www.apache.org/licenses/LICENSE-2.0
 *  *
 *  * Unless required by applicable law or agreed to in writing, software
 *  * distributed under the License is distributed on an "AS IS" BASIS,
 *  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  * See the License for the specific language governing permissions and
 *  * limitations under the License.
 *
 */

import net.projectmonkey.object.mapper.construction.PopulationContext;
import net.projectmonkey.object.mapper.context.ExecutionContext;
import net.projectmonkey.object.mapper.ObjectMappingService;
import net.projectmonkey.object.mapper.construction.type.DestinationTypeProvider;

/**
 * Converter which handles types mapped in the configured {@link net.projectmonkey.object.mapper.construction.type.DestinationTypeProvider}
 * If a destination exists and is of the correct mapped destination type then the source will be merged into the destination
 *
 * otherwise a new instance of the required destination type will be constructed and populated.
 *
 * @author Andy Moody
 */
public class MappedDestinationTypeConverter implements Converter<Object, Object>
{

	public static final MappedDestinationTypeConverter INSTANCE = new MappedDestinationTypeConverter();

	private MappedDestinationTypeConverter(){}

	@Override
	public Object convert(final PopulationContext<Object, Object> context)
	{
		Class<?> newDestinationType = getDestinationTypeProvider().resolveDestinationType(context);
		Object destination = context.getDestination();

		ObjectMappingService mappingService = ExecutionContext.getMappingService();
		Object source = context.getSource();
		if(destination != null && newDestinationType.isAssignableFrom(destination.getClass()))
		{
			mappingService.merge(source, destination);
		}
		else
		{
			destination = mappingService.map(source, newDestinationType);
		}
		return destination;
	}

	@Override
	public boolean canConvert(final PopulationContext<?, ?> context)
	{
		return getDestinationTypeProvider().resolveDestinationType(context) != context.getDestinationType();
	}

	private DestinationTypeProvider getDestinationTypeProvider()
	{
		return ExecutionContext.getConfiguration().getDestinationTypeProvider();
	}
}
