package net.projectmonkey.object.mapper.construction.type;

/*
 *
 *  * Copyright 2012 the original author or authors.
 *  *
 *  * Licensed under the Apache License, Version 2.0 (the "License");
 *  * you may not use this file except in compliance with the License.
 *  * You may obtain a copy of the License at
 *  *
 *  *      http://www.apache.org/licenses/LICENSE-2.0
 *  *
 *  * Unless required by applicable law or agreed to in writing, software
 *  * distributed under the License is distributed on an "AS IS" BASIS,
 *  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  * See the License for the specific language governing permissions and
 *  * limitations under the License.
 *
 */

import net.projectmonkey.object.mapper.util.CollectionUtil;

import java.util.*;

/**
 * Simple Generic type resolver for some common list types
 *
 * @author Andy Moody
 */
public class ListGenericTypeResolver implements GenericTypeResolver<List<?>>
{

	private Class<?> destinationElementType;

	public ListGenericTypeResolver(final Class<?> destinationElementType)
	{
		this.destinationElementType = destinationElementType;
	}

	@Override
	public List<Class<?>> getSourceTypeArguments(final List<?> source)
	{
		Class<?> elementType = Object.class;
		if(CollectionUtil.hasElements(source))
		{
			elementType = source.get(0).getClass();
		}
		return Arrays.<Class<?>>asList(elementType);
	}

	@Override
	public List<Class<?>> getDestinationTypeArguments(final List<?> source)
	{
		return Arrays.<Class<?>>asList(destinationElementType);
	}

	@Override
	public Class<? extends List<?>>[] getApplicableSourceTypes()
	{
		return new Class[]{List.class, ArrayList.class, LinkedList.class, Vector.class};
	}

}
